# Parameters of discretization 
param Nt := 250*2; 
param Nx := 50; 
param L := 30; 
param tf := 50; 
param dx := 2*L/Nx; 
param dt := tf/Nt; 
param ymax := 0.6; 
set kx ordered := 0..Nx; 
set kt ordered := 0..Nt; 
param x{j in kx}; 
let {j in kx} x[j] := -L+j*dx; 
param t{i in kt\}; 
let {i in kt} t[i] := i*dt;

# Parameters of control problem
param u1max := 5; # a_max 
param u1min := -5; # a_min 
param u2max := 1; # tet_max 
param u2min := 0; # tet_min 
param tet0 := 0.7;
param tetf := 0.4; 
param a0 := -1; 
param af := -0.2; 
param Ktet := 0.01; 
param Ka := 0.01; 
param x1 := 0;

# Parameters of initial data 
param A0 := sqrt(-a0); 
param c0 := -A0*sqrt(2)*(1/2-tet0); 
param y0 {j in kx};
let {j in 0..Nx/2} y0[j] := 0; 
let {j in Nx/2+1..Nx} y0[j] := tet0-0.01;

# Parameters of desired solution
param Af := sqrt(-af); 
param cf := -Af*sqrt(2)*(1/2-tetf); 
param yobj{i in kx}; 
param x2 := x1+cf*Nt*dt;
let {j in kx} yobj[j] := 0.35 ;

# Parametre for initialisation \par
# (0 => rien; 1 => constant; 2 => init.txt) 
param Init_Type = 1;


# Declare the variables and their bounds 
var a{i in kt} >= u1min, <= u1max; 
var tet{i in kt} >= u2min, <= u2max; 
var y{i in kt, j in kx} >= 0, <= 1;

# Specify the objective function
minimize obj: sum{j in kx} ( ( y[Nt,j] - yobj[j])\textasciicircum{}2
+ Ktet*sum{j in kt diff{0}} (tet[j]-tet[j-1])\textasciicircum{}2
+ Ka*sum{j in kt diff{0}} (a[j]-a[j-1])\textasciicircum{}2;

# Contraints of the control 
subject to c1: tet[0] - tet0 = 0; 
subject to c2: a[0] - a0 = 0; 
subject to c3: tet[Nt] - tetf = 0; 
subject to c4: a[Nt] - af = 0; 

# Initial data
subject to i1 {j in kx}: y[0,j] = y0[j] ; 

# Dynamical constraints (Backward ) 
subject to d1 {i in kt diff{Nt}, j in kx diff {0,Nx}}:
  (y[i+1,j] - y[i,j]) - 1/2*dt*( y[i+1,j+1]-2*y[i+1,j]+
  y[i+1,j-1]+y[i,j+1]-2*y[i,j]+y[i,j-1])/dx\textasciicircum{}2-
  1/2*dt*( a[i+1]*y[i+1,j]*(1 - y[i+1,j]) *(tet[i+1]- y[i+1,j])+
  a[i]*y[i,j]*(1 - y[i,j]) *(tet[i]- y[i,j])) = 0;

# Boundary constraints
subject to b1 {i in kt diff{0}}: y[i,1] - y[i,0] = 0; 
subject to b2 {i in kt diff{0}}: y[i,Nx] - y[i,Nx-1] = 0;


# Initialization of state and control variables
if (Init_Type == 1) then { 
  let {j in kt} tet[j] := tetf;
  let {j in kt} a[j] := 0;
  for {i in kt diff{0},j in kx}{
   let y[i,j] := ymax*exp(Af*(-L+j*dx-cf*i*dt)/sqrt(2))/
     (1 + exp(Af*(-L+j*dx-cf*i*dt)/sqrt(2)));
  }
};
# Initialisation avec \textquotedbl{}init.txt\textquotedbl{}(only state and control)
if (Init_Type == 2) then { 
  read {i in kt} (a[i],tet[i]) < init.txt; 
 read {i in kt, j in kx} (y[i,j]) < init.txt; 
};

